/*******************************************************************************

   HIDROMETER_EE

   C/ SENSOR DE VAZO YF_S201

   MICROCONTROLADOR PIC16F628A

   AUTOR: CLAUDIO LARIOS

   DATA 13/03/2019

   APLICAO: MEDIR A VAZO (L/MIN) E O CONSUMO (LITROS)

   ULTIMA MEDIDA DE CONSUMO SER SALVA NA EEPROM INTERNA
   (QUANDO VCC FOR DESLIGADO, SE USADO O ESQUEMA PROPOSTO)

   ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG PICSOURCE (LARIOS.TECNOLOGIA.WS)

   USO DIDTICO

*******************************************************************************/

#include <hidrometer_EE_v3.h>// arquivo de cabeceira
#include <REG16F628A.h>//endereo dos registradores internos do PIC
#define use_portb_lcd //portb ligada nos pinos do LCD
#include <LCDD.C>//arquivo do driver para LCD comum
#rom 0x2100 = {0,0,0,0}

#bit sinal = 0x5.2  //define o pino 1 como entrada do sinal do sensor YF-S201
#bit v_vcc = 0x5.3  //define o pino 2 como entrada sensora de falta de tenso


//variveis globais

unsigned int1 trava1=0; //flag gerenciador de comparao dos centilitros
unsigned int1 trava2=0;//flag limitador de nica execuo ps mudana de estado
unsigned int1 vz_ok=0;//flag indicador de leitura de vazo 'ok'

unsigned int8 cont1=0;//auxiliar

float consumo=0;     //contador de consumo em litros
float consumo_ant=0; //auxiliar
float consumo_at=0;  //auxiliar
float vazao=0;       //contador de vazo em litros/min


/*******************************************************************************
   Rotina de Apresentao de Resultados no LCD
*******************************************************************************/
void mostra_lcd(){

      lcd_gotoxy(1,1);//linha 1
      printf(lcd_putc,"Vaz= %04.2f L/min ",vazao);//mostra a vazo na linha 1

      lcd_gotoxy(1,2);//linha 2

      printf(lcd_putc,"C= %010.2f L  ",consumo);//mostra consumo na linha 2

}

/*******************************************************************************
       Rotina de Leitura de Consumo
   Conforme datasheet do YF-S201, so produzidos 450 pulsos/Litro.
   O consumo ser a soma de cada pulso em um registrador de consumo.
   O valor de cada pulso ser: 1/450= 0.0022222(mas na prtica funciona melhor
   a relao 1/400= 0,0025 ou mais prximo deste valor).
   Esta relao pulsos por litros poder variar de uma unidade para outra.
   Portanto, para uma melhor preciso, faa teste de vazo, medindo o valor em
   litros com um recepiente graduado e compare com a medio obtida no LCD.
   Caso ocorra diferena, poder alterar o valor do 'define'
   'FATOR_LITRO_POR_PULSO' para mais ou menos  e refaa os testes, at obter uma
   preciso desejada.
   Obs. No espere muita preciso com este sensor ( aprox. 10% de tolerncia)
   Em vazes diferentes tambm ocorrer diferenas na medio feita com um
   recepiente graduado.
*******************************************************************************/

void le_consumo(){
#define FATOR_LITRO_POR_PULSO  0.0025 //defina por teste prtico

  if(!sinal) trava2=0;//se entrada do sensor ficou '0' reseta o flag.
  if(sinal && !trava2){//se sinal do sensor ficar '1'
        trava2=1;//seta flag
        consumo+=FATOR_LITRO_POR_PULSO;
  }
}
/*******************************************************************************
       Rotina de Obteno da Vazo
   Metodo: Ao invs de medir a frequncia do sinal, usamos o valor acumulado
   no contador de consumo para obter a vazo.
   Basta seguir a formula abaixo:
   Leitura anterior - Leitura Atual (obtida aps um segundo) X 60
*******************************************************************************/

void le_vazao(){
  if(vz_ok){//testa flag
    vazao= (consumo_at - consumo_ant) * 60;//consumo atual - anterior * 60
    vz_ok=0;//reseta flag
  }
}
/*******************************************************************************
             Rotina de Interrupo do Timer 1
   A cada 125 milissegundos (4 * 1/xtal * prescaller * tmr1h/tmr1l)
   onde:
   'xtal' igual 16mhz;
   'prescaller' igual a 8;
   'tmr1h/tmr1l' com carga inicial para obter contagem de 50000.
   O flag 'vz_ok' ser setado na oitava interrupo do timer1 ( a cada segundo).
   O valor ADJ  uma tentativa de compensao por delays adicionados pelas
   instrues de salvamento de registradores na ocasio da interrupo.
*******************************************************************************/
#int_timer1
void isr_timer1(){
#define  TEMPO_DESEJADO_MS  125
#define  PRESCALLER_DIV   8
#define  FREQ  16000000
#define  FATOR_DIV_CLOCK_PIC 4
#define  NUMERADOR  (FREQ * TEMPO_DESEJADO_MS)
#define  DENOMINADOR  (PRESCALLER_DIV * FATOR_DIV_CLOCK_PIC * 1000)
#define  CARGA (0xffff - (NUMERADOR/DENOMINADOR))
#define  V_ADJ  5 // valor para correo de delays indesejados
#define  V_CALC    CARGA + V_ADJ
#define   CARGA_TMR1  V_CALC   //valor p/ obter tempo de 125 mseg por interrupo

  TMR1ON =0;//desliga TMR1
  TMR1L= CARGA_TMR1;//carga do timer 1
  TMR1H= CARGA_TMR1>>8;
  TMR1ON =1;//liga TMR1
  if(vz_ok)return;
  if(!trava1){
     trava1=1;//seta trava1
     consumo_ant=consumo;//salva valor do consumo antes do delay de 1 segundo
  }

  cont1++;
  if(cont1>8){//Se contador =9, temos 1 segundo
  consumo_at=consumo;//salva o valor do consumo aps 1 segundo

  vz_ok=1;//seta flag para notificar que o valor da vazo foi obtido
  cont1=0;//zera contador
  trava1=0;//reseta trava1
  }


}
/*******************************************************************************
       Rotina de Recuperao da Contagem Salva na EEprom Interna
*******************************************************************************/

void recupera_contagem(){//recupera a contagem
 unsigned int32 t;
 float y;
   t= (long)read_eeprom(3);
   t+=(long) read_eeprom(2)<<8;
   t+=(long) read_eeprom(1)<<16;
   t+=(long)read_eeprom(0)<<24;

   y=(float)t;
   consumo=y * 0.001;//retira o acrscimo de salvamento ps virgula

}
/*******************************************************************************
       Rotina de Salvamento da Contagem na EEprom Interna
*******************************************************************************/
void salva_contagem(){//salva a contagem
unsigned int32 x;
float z;
  z=consumo * 1000.0; //salva tambm a parte ps vrgula
  x=(long)z;
  write_eeprom(3, x);
  write_eeprom(2, x>>8);
  write_eeprom(1, x>>16);
  write_eeprom(0, x>>24);
}
/*******************************************************************************
       Rotina Main - Configuraes dos registradores e Inicializaes
*******************************************************************************/
void main() {
   delay_ms(50);
   CMCON =0X7;//Desliga comparadores internos
   lcd_init();
   printf(lcd_putc,"  Hidrometer ");//mostra o nome
   lcd_gotoxy(1,2);//linha 2
   printf(lcd_putc,"  by Picsource  ");// o blog
   delay_ms(2000);
   lcd_putc("\f");//apaga tela
   recupera_contagem();
   T1CON =0X30;//timer 1 habilitado div/8
   TMR1L= CARGA_TMR1;
   TMR1H= CARGA_TMR1>>8;
   TMR1IF=0;
   TMR1ON=1;//liga timer 1
   GIE=1;//liga geral das interrupes
   PEIE=1;//liga chave das interrupes perifricas
   TMR1IE=1;//liga interrupo do timer 1
/*******************************************************************************
       Loop Eterno
*******************************************************************************/
   for(;;){

     if(!v_vcc){//testa se houve queda no fornecimento de energia
       salva_contagem();//se houve,salva o valor do consumo na EEprom
       while(!v_vcc);//at acabar carga dos capacitores da fonte
     }
     le_vazao();//obtm o valor da vazo
     le_consumo();//obtm o valor do consumo (litros)
     mostra_lcd();//atualiza resultados no LCD

   }//for(;;)
}//main()
